import java.util.*;

public abstract class Constraint {

    Problem pb;
    ArrayList<IntVar> variables;
    LinkedList<IntVar> removals, newUPB, newLWB, instantiations;
    boolean onStack, init;
    String name;
    

    public Constraint(Problem pb){
	this.pb        = pb;
	variables      = new ArrayList<IntVar>();
	removals       = new LinkedList<IntVar>();
	newUPB         = new LinkedList<IntVar>();
	newLWB         = new LinkedList<IntVar>();
	instantiations = new LinkedList<IntVar>();	
	onStack        = false;
	init           = true;
    }

    void addVariables(IntVar[] v){
	for (IntVar x : v) addVariable(x);
    }

    void addVariable(IntVar x){
	variables.add(x);
	x.constraints.add(this);
    }

    abstract void propagateInit();
    
    abstract void propagateRemoval(IntVar x, int value);
    
    abstract void propagateUPB(IntVar x);
    
    abstract void propagateLWB(IntVar x);
    
    abstract void propagateInstantiate(IntVar x);

    abstract boolean isLegal();

    public String toString(){
	String s = name;
	for (IntVar v : variables) s = s +" "+ v;
	if (onStack) s = s +" onStack";
	return s;
    }
    
}
