import java.util.*;

public class Domain {

    Problem pb;
    int[] value;
    int[] location;
    int lwb, upb, n;
    ReversibleInt last, min, max;
    

    public Domain(Problem pb,int lwb, int upb){
	this.pb   = pb;
	this.lwb  = lwb;
	this.upb  = upb;
	n         = upb - lwb + 1;
	value     = new int[n];
	location  = new int[n];
	last      = new ReversibleInt(pb,n-1);
	min       = new ReversibleInt(pb,lwb);
	max       = new ReversibleInt(pb,upb);
	for (int i=0;i<n;i++) value[i] = i;
	for (int i=0;i<n;i++) location[i] = i;
	
    }

    void swap(int[] array,int i,int j){
	int temp = array[i];
	array[i] = array[j];
	array[j] = temp;
    }

    boolean remove(int x){
	if (x < lwb || x > upb || !contains(x)) return false;
	int v = x - lwb;
	int w = value[last.getValue()];
	swap(value,last.getValue(),location[v]);
	location[w] = location[v];
	location[v] = last.getValue();
	last.setValue(last.getValue() - 1);
	if (isEmpty()) throw new CPException("Domain is empty.");
	if (x == min.getValue()) min.setValue(getNewMin());
	if (x == max.getValue()) max.setValue(getNewMax());
	return true;
    }

    int size(){return last.getValue() + 1;}

    boolean isEmpty(){return size() == 0;}

    boolean removeAllBut(int x){
	if (x < lwb || x > upb || !contains(x))
	    throw new CPException("The value "+ x +" is not in the domain.");
	if (size() == 1) return false;
	int v = x - lwb;
	int w = value[0];
	swap(value,0,location[v]);
	location[w] = location[v];
	location[v] = 0;
	last.setValue(0);
	min.setValue(x);
	max.setValue(x);
	if (isEmpty()) throw new CPException("Domain is empty.");
	return true;
    }	

    
    int getNewMin(){
	if (isEmpty()) throw new CPException("Domain is empty.");
	for (int i=min.getValue()-lwb;i<n;i++)
	    if (location[i] <= last.getValue()) return i + lwb;
	return Integer.MAX_VALUE;
    }

    int min(){return min.getValue();}
    
    int getNewMax(){
	if (isEmpty()) throw new CPException("Domain is empty.");
	for (int i=max.getValue()-lwb;i>=0;i--)
	    if (location[i] <= last.getValue()) return i + lwb;
	return Integer.MIN_VALUE;
    }

    int max(){return max.getValue();}

    boolean removeBelow(int x){
	boolean changed = false;
	for (int v=min();v<x;v++)
	    changed = remove(v) || changed;
	if (isEmpty()) throw new CPException("Domain is empty.");
	return changed;
    }

    boolean removeAbove(int x){
	boolean changed = false;
	for (int v=max();v>x;v--)
	    changed = remove(v) || changed;
	if (isEmpty()) throw new CPException("Domain is empty.");
	return changed;
    }
    
    void reset(){
	last.setValue(n-1);
    }

    boolean contains(int x){
	if (x < lwb || x > upb) return false;
	int v = x - lwb;
	return location[v] <= last.getValue();
    }

    int getLastValue(){
	return value[last.getValue()] + lwb;
    }
    //
    // only safe to use when domain is a singleton
    //

    public String toString(){
	if (isEmpty()) return "[]";
	String s = "["+ (lwb + value[0]);
	for (int i=1;i<size();i++)
	    s = s +","+ (lwb + value[i]);
	return s +"]";
    }

    int[] enumerate(){
	int[] values = new int[size()];
	for (int i=0;i<size();i++)
	    values[i] = lwb + value[i];
	return values;
    }
}
