import java.util.*;

public class IntVar extends Var implements Iterable<Integer> {
    
    String name;
    ArrayList<Constraint> constraints;
    LinkedList<Integer> deletions;
    Problem pb;
    Domain domain;    

    public IntVar(String name,int lwb, int upb,Problem pb){
	this.name   = name;
	this.pb     = pb;
	domain      = new Domain(pb,lwb,upb);
	constraints = new ArrayList<Constraint>();
	deletions   = new LinkedList<Integer>();
	pb.variables.add(this);
    }

    int size(){return domain.size();}

    boolean contains(int x){return domain.contains(x);}
    
    boolean remove(int x){
	boolean changed = domain.remove(x);
	if (changed) changeRemoval(x);
	return changed;
    }
    
    boolean instantiate(int x){
	boolean changed = domain.removeAllBut(x);
	if (changed) changeInstantiate();
	return changed;
    }
    
    boolean isInstantiated(){return domain.size() == 1;}

    int min(){return domain.min();}

    int max(){return domain.max();}

    boolean removeAbove(int x){
	boolean changed = domain.removeAbove(x);
	if (changed) changeUPB();
	return changed;
    }

    boolean removeBelow(int x){
	boolean changed = domain.removeBelow(x);
	if (changed) changeLWB();
	return changed;
    }

    int getValue(){
	if (!isInstantiated()) throw new IntVarException("Not instantiated");
	return domain.getLastValue();
    }

    void changeRemoval(int value){	   
	if (size() == 1) changeInstantiate();
	if (value > max()) changeUPB();
	if (value < min()) changeLWB();
	if (min() < value && value < max()){
	    deletions.add(value);
	    for (Constraint c : constraints){
		if (!c.onStack){
		    pb.revisionStack.push(c);
		    c.onStack = true;
		}
		if (!c.removals.contains(this)) c.removals.add(this);
	    }
	}
    }

    void changeInstantiate(){
	for (Constraint c : constraints){
	    if (!c.onStack){
		pb.revisionStack.push(c);
		c.onStack = true;
	    }
	    if (!c.instantiations.contains(this)) c.instantiations.add(this);
	}
    }
	
    void changeUPB(){
	if (size() == 1)
	    changeInstantiate();
	else
	    for (Constraint c : constraints){
		if (!c.onStack){
		    pb.revisionStack.push(c);
		    c.onStack = true;
		}
		if (!c.newUPB.contains(this)) c.newUPB.add(this);
	    }
    }

    void changeLWB(){
	if (size() == 1)
	    changeInstantiate();
	else
	    for (Constraint c : constraints){
		if (!c.onStack){
		    pb.revisionStack.push(c);
		    c.onStack = true;
		}
		if (!c.newLWB.contains(this)) c.newLWB.add(this);
	    }
    }
    
    int getLwb(){return domain.lwb;}
    
    int getUpb(){return domain.upb;}
    
    public String toString(){
	return name +": "+ domain;
    }
    
    public Iterator<Integer> iterator(){
	DomainIterator diter = new DomainIterator(this);
	return (Iterator<Integer>) diter;
    }
}
