//import java.util.*;
//
// x = max(v) where v is IntVar[] and x is IntVar
//

public class Max extends Constraint {

    IntVar max;
    IntVar[] v;
    ReversibleInt  indexOfLargest;
    ReversibleInt  largestMax;
    ReversibleInt  largestInst;
    ReversibleInt  largestMin;
    ReversibleBool entailed;
    ReversibleInt[] support; // support[x] = i where v[i] contains x
    int n, m, maxLwb, maxUpb;

    public Max(Problem pb,IntVar max,IntVar[] v){
	super(pb);
	this.max        = max;
	this.v          = v;
	this.n          = v.length;
	name            = "Max";
	largestInst     = new ReversibleInt(pb,Integer.MIN_VALUE);
	largestMin      = new ReversibleInt(pb,Integer.MIN_VALUE);
	largestMax  = new ReversibleInt(pb,Integer.MIN_VALUE);
	indexOfLargest  = new ReversibleInt(pb,-1);
	entailed        = new ReversibleBool(pb,false);
	maxLwb          = max.getLwb();
	maxUpb          = max.getUpb();
	m               = maxUpb - maxLwb + 1;
	support         = new ReversibleInt[m];

	addVariables(v);
	addVariable(max);
	for (int i=0;i<m;i++) support[i] = new ReversibleInt(pb,0);
    }

    void propagateInit(){
	findLargestMax();
	findLargestMin();
	max.removeAbove(largestMax.getValue());
	max.removeBelow(largestMin.getValue());
	for (Integer x : max) 
	    if (!supported(x)) max.remove(x);
	for (IntVar x : v) x.removeAbove(max.max());
    }

    void propagateRemoval(IntVar x, int value){
	if (!x.equals(max) && !supported(value)) max.remove(value);
    }
    
    void propagateUPB(IntVar x){
	if (x.equals(max)){
	    if (max.max() < largestMax.getValue()){
		for (IntVar y : v) y.removeAbove(max.max());
		findLargestMax();
	    }
	}
	if (!x.equals(max)){
	    if (x.equals(v[indexOfLargest.getValue()])){
		findLargestMax();
		max.removeAbove(largestMax.getValue());
	    }
	    for (int val : max)
		if (val > x.max() && !supported(val)) max.remove(val);
	}
    }
    
    void propagateLWB(IntVar x){
	if (!x.equals(max)){
	    max.removeBelow(x.min());
	    for (int val : max)
		if (val < x.min() && !supported(val)) max.remove(val);
	}
    }
    
    void propagateInstantiate(IntVar x){
	if (x.equals(max)){
	    if (max.max() < largestMax.getValue()){
		for (IntVar w : v) w.removeAbove(max.getValue());
		findLargestMax();
	    }
	    if (uniqueLargest()){
		v[indexOfLargest.getValue()].instantiate(max.getValue());
		largestInst.setValue(max.getValue());
	    }
	}
	if (!x.equals(max)){
	    max.removeBelow(x.getValue());
	    if (x.equals(v[indexOfLargest.getValue()]) && x.getValue() < largestMax.getValue()){
		findLargestMax();
		max.removeAbove(largestMax.getValue());
	    }
	    largestInst.setValue(Math.max(largestInst.getValue(),x.getValue()));
	    for (int val : max) if (!supported(val)) max.remove(val);
	}
	entailed.setValue(max.isInstantiated() && max.getValue() == largestInst.getValue());
    }	

    boolean isLegal(){
	for (int x : max)
	    if (!supported(x)) return false;
	return max.max() == largestMax.getValue();
    }

    void findLargestMax(){
	int largest = Integer.MIN_VALUE;
	int index   = 0;
	for (int i=0;i<n;i++)
	    if (v[i].max() > largest){
		largest = v[i].max();
		index = i;
	    }
	largestMax.setValue(largest);
	indexOfLargest.setValue(index);
    }

    void findLargestMin(){
	for (int i=0;i<n;i++)
	    largestMin.setValue(Math.max(largestMin.getValue(),v[i].min()));
    }
    //
    // Find the largest lower bound in v.
    // max cannot be smaller than that!
    //

    boolean uniqueLargest(){
	int largest = largestMax.getValue();
	for (int i=indexOfLargest.getValue()+1;i<n;i++)
	    if (v[i].max() == largest) return false;
	return true;
    }
    //
    // Is the largest value in v unique?
    // Search starts after location of first largest found
    // Must be called after clean call to findLargest
    //

    boolean supported(int w){
	for (int i=support[w-maxLwb].getValue();i<n;i++)
	    if (v[i].contains(w)){
		support[w-maxLwb].setValue(i);
		return true;
	    }
	support[w-maxLwb].setValue(n);
	return false;
    }
    //
    // Is the value w in max.domain have support in array v?
    //
	    
}
