package weeSeepy.constraints;

import java.util.*;
import weeSeepy.problem.*;
import weeSeepy.variables.*;
import weeSeepy.exceptions.*;

public abstract class Constraint {

    public Problem pb;
    public ArrayList<IntVar> variables;
    public LinkedList<IntVar> removals, newUPB, newLWB, instantiations;
    public boolean onQueue, init, isReified, posted, isOpposite;
    public Constraint opposite;
    public IntVar b; // refication boolean variable
    String name;
    

    public Constraint(Problem pb){
	this.pb        = pb;
	variables      = new ArrayList<IntVar>();
	removals       = new LinkedList<IntVar>();
	newUPB         = new LinkedList<IntVar>();
	newLWB         = new LinkedList<IntVar>();
	instantiations = new LinkedList<IntVar>();	
	onQueue        = false;
	init           = true;
	isReified      = false;
	posted         = false;
	opposite       = null;
	b              = null;
    }

    public void addVariables(IntVar[] v){
	for (IntVar x : v) addVariable(x);
    }

    public void addVariable(IntVar x){
	variables.add(x);
    }

    public boolean revise(){
	if (pb.trace) System.out.println("revise: "+ this);
	if (isReified){
	    if (pb.trace) System.out.println("reifiedPropagation: "+ this +" with "+ b);
	    try {reifiedPropagation();}
	    catch (CPException e) {return false;}
	    return true;
	}
	if (init){
	    try {propagateInit();}
	    catch (CPException e) {return false;}
	    init = false;
	}
	while (removals.size() > 0 || newUPB.size() > 0 || newLWB.size() > 0 ||  instantiations.size() > 0){
	    while (!removals.isEmpty()){
		IntVar v = removals.remove();
		while (!v.deletions.isEmpty()){
		    int x = v.deletions.remove();
		    if (pb.trace) System.out.println("removeProp: "+ v +" "+ x +" "+ this);
		    try {propagateRemoval(v,x);}
		    catch (CPException e) {return false;}
		}
	    }
	    while (!newUPB.isEmpty()){
		IntVar v = newUPB.remove();
		if (pb.trace) System.out.println("newUPBProp: "+ v +" "+ this);
		try {propagateUPB(v);}
		catch (CPException e) {return false;}
	    }
	    while (!newLWB.isEmpty()){
		IntVar v = newLWB.remove();
		if (pb.trace) System.out.println("newLWBProp: "+ v +" "+ this);
		try {propagateLWB(v);}
		catch (CPException e) {return false;}
	    }
	    while (!instantiations.isEmpty()){
		IntVar v = instantiations.remove();
		if (pb.trace) System.out.println("instantiationProp: "+ v +" "+ this);
		try {propagateInstantiate(v);}
		catch (CPException e) {return false;}
	    }
	}
	return true;
    }

    public void reify(){
	b = new IntVar("b",0,1,pb);
	b.constraints.add(this);
	isReified = true;
	isOpposite = false;
	opposite = this.makeOpposite();
	opposite.b = b;
	opposite.isReified = true;
	opposite.isOpposite = true;
	opposite.opposite = this;
    }	

    boolean localRevision(){
	if (pb.trace) System.out.println("localRevison: "+ this);
	boolean satisfied = true;
	pb.propagationOn = false;
	 pb.pushWorld();
	  try {propagate();}
	  catch (CPException e) {satisfied = false;}
	  if (pb.trace) System.out.println("RESULT: "+ this +" -> "+ satisfied);
	 pb.popWorld();
	pb.propagationOn = true;
	return satisfied;
    }

    void reifiedPropagation(){
	Constraint c = isOpposite ? opposite : this;
	if (b.contains(0) && b.contains(1)){
	    if (!c.localRevision()) b.remove(1);
	    if (!c.opposite.localRevision()) b.remove(0);
	}
	if (b.isInstantiated() && b.getValue() == 1) c.propagate();
	if (b.isInstantiated() && b.getValue() == 0) c.opposite.propagate();
    }

    public abstract void propagate();
    
    public abstract void propagateInit();
    
    public abstract void propagateRemoval(IntVar x, int value);
    
    public abstract void propagateUPB(IntVar x);
    
    public abstract void propagateLWB(IntVar x);
    
    public abstract void propagateInstantiate(IntVar x);

    public abstract boolean isEntailed();

    public abstract Constraint makeOpposite();

    public abstract boolean isLegal();

    public String toString(){
	String s = name;
	for (IntVar v : variables) s = s +" "+ v;
	if (onQueue) s = s +" on Queue";
	if (!onQueue) s = s +" not on Queue";
	return s;
    }
    
}
