package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

public class Equals extends BinaryConstraint {

    public Equals(Problem pb,IntVar x, IntVar y){
	super(pb,x,y);
	name = "Equals";
	assert !x.equals(y) : "x and y must be different "+ this;
    }

    public void propagate(){
	for (int i : x)
	    if (!y.contains(i)) x.remove(i);
	for (int i : y)
	    if (!x.contains(i)) y.remove(i);
    }

    public void propagateInit(){
	propagate();
    }

    public void propagateRemoval(IntVar v, int value){
	x.remove(value);
	y.remove(value);
    }
    
    public void propagateUPB(IntVar v){
	x.removeAbove(y.max());
	y.removeAbove(x.max());
    }
    
    public void propagateLWB(IntVar v){
	x.removeBelow(y.min());
	y.removeBelow(x.min());
    }
    
    public void propagateInstantiate(IntVar v){
	if (x.isInstantiated()) y.instantiate(x.getValue());
	if (y.isInstantiated()) x.instantiate(y.getValue());
    }

    public boolean isEntailed(){
	return x.isInstantiated() && y.isInstantiated();
    }

    public Constraint makeOpposite(){
	return new NotEquals(pb,x,y);
    }
    //
    // not(x = y) == x != y
    //
    
    public boolean isLegal(){
	for(int i : x)
	    if (!y.contains(i))
		return false;
	for(int i : y)
	    if (!x.contains(i))
		return false;
	return true;
    }    
}
