package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

//
// x > y + c
// where c is positive
//

public class GreaterThan extends BinaryConstraint {

    int c;

    public GreaterThan(Problem pb,IntVar x,IntVar y){
	super(pb,x,y);
	name = "GreaterThan";
	c = 0;
    }

    public GreaterThan(Problem pb,IntVar x,IntVar y,int c){
	super(pb,x,y);
	name = "GreaterThan";
	this.c = c;
	assert c >= 0 : "c must be positive " + this;
    }

    public GreaterThan(Problem pb,IntVar x,int y){
	super(pb,x,new IntVar("anon",y,y,pb));
	name = "GreaterThan";
	c = 0;
    }

    public void propagateInit(){
	propagate();
    }

    public void propagate(){
	x.removeBelow(y.min()+1+c);
	y.removeAbove(x.max()-1-c);
    }
    
    public void propagateRemoval(IntVar v, int value){}
    
    public void propagateUPB(IntVar v){propagate();}
    
    public void propagateLWB(IntVar v){propagate();}
    
    public void propagateInstantiate(IntVar v){propagate();}

    public boolean isEntailed(){
	return x.min() > y.max()+c;
    }

     public Constraint makeOpposite(){
	 return new LessThanOrEqual(pb,x,y,c);
    }
    //
    // not(x > y + c)) ==  x <= y + c
    //

    public boolean isLegal(){
	return x.max() > y.max() + c && x.min() > y.min() + c;
    }
}
