package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

//
// x >= y + c
// where c is positive
//

public class GreaterThanOrEqual extends BinaryConstraint {

    int c;

    public GreaterThanOrEqual(Problem pb,IntVar x,IntVar y){
	super(pb,x,y);
	name = "GreaterThanOrEqual";
	c = 0;
    }

    public GreaterThanOrEqual(Problem pb,IntVar x,IntVar y,int c){
	super(pb,x,y);
	name = "GreaterThanOrEqual (with c = "+ c +") ";
	this.c = c;
	assert c >= 0 : "c must be positive " + this;
    }

    public GreaterThanOrEqual(Problem pb,IntVar x,int y){
	super(pb,x,new IntVar("anon",y,y,pb));
	name = "GreaterThanOrEqual";
	c = 0;
    }

    public void propagateInit(){
	propagate();
    }

    public void propagate(){
	x.removeBelow(y.min()+c);
	y.removeAbove(x.max()-c);
    }
    
    public void propagateRemoval(IntVar v, int value){}
    
    public void propagateUPB(IntVar v){propagate();}
    
    public void propagateLWB(IntVar v){propagate();}
    
    public void propagateInstantiate(IntVar v){propagate();}

    public boolean isEntailed(){
	return x.min() >= y.max()+c;
    }

     public Constraint makeOpposite(){
	 return new LessThan(pb,x,y,c);
    }
    //
    // not(x >= y + c)) ==  x < y + c
    //

    public boolean isLegal(){
	return x.max() >= y.max() + c && x.min() >= y.min() + c;
    }
}
