package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

//
// x <= y + c
// where c is positive
//

public class LessThanOrEqual extends BinaryConstraint {

    int c;

    public LessThanOrEqual(Problem pb,IntVar x,IntVar y){
	super(pb,x,y);
	name = "LessThanOrEqual";
	c = 0;
    }

    public LessThanOrEqual(Problem pb,IntVar x,IntVar y,int c){
	super(pb,x,y);
	name = "LessThanOrEqual";
	this.c = c;
	assert c >= 0 : "c must be positive " + this;
    }

    public LessThanOrEqual(Problem pb,IntVar x,int y){
	super(pb,x,new IntVar("anon",y,y,pb));
	name = "LessThaOrEqual";
	c = 0;
    }

    public void propagateInit(){
	propagate();
    }

    public void propagate(){
	x.removeAbove(y.max()+c);
	y.removeBelow(x.min()-c);
    }
    
    public void propagateRemoval(IntVar v, int value){}
    
    public void propagateUPB(IntVar v){propagate();}
    
    public void propagateLWB(IntVar v){propagate();}
    
    public void propagateInstantiate(IntVar v){propagate();}

    public boolean isEntailed(){
	return x.max() <= y.min()+c;
    }

     public Constraint makeOpposite(){
	 return new GreaterThan(pb,x,y,c);
    }
    //
    // not(x <= y + c) ==  x > y + c
    //

    public boolean isLegal(){
	return x.max() <= y.max() + c && x.min() <= y.min() + c;
    }
}
