//
// sum(a[i].x[i]) >= c
//

package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

public class SumGEQ extends Constraint {

    int c, n;
    int[] a;
    IntVar[] x;

    public SumGEQ(Problem pb, int[] a, IntVar[] x, int c){
	super(pb);
	this.c = c;
	this.a = a;
	this.x = x;
	this.n = x.length;
	name   = "SumGEQ (with c = "+ c +"): ";
	addVariables(x);
	assert allPositive() : "all coeffs must be > 0 "+ this;
    }

    public void propagate(){
	int sumOfMaxs = 0;
	for (int i=0;i<n;i++) sumOfMaxs = sumOfMaxs + a[i] * x[i].max();
	int slack = c - sumOfMaxs; 
	for (int j=0;j<n;j++){
	    int S_j = slack + a[j] * x[j].max(); // max slack on a[j]*x[j].max()
	    x[j].removeBelow(S_j/a[j]); // can have no less than this!
	}
    }

    public void propagateInit(){
	propagate();
    }
    
    public void propagateRemoval(IntVar x, int value){}
    
    public void propagateUPB(IntVar x){
	propagate();
    }
    
    public void propagateLWB(IntVar x){
	propagate();
    }
    
    public void propagateInstantiate(IntVar x){
	propagate();
    }
    
    public boolean isEntailed(){
	return false;
    }

    public Constraint makeOpposite(){return new Null(pb);}

    public boolean isLegal(){
	return true;
    }

    private boolean allPositive(){
	for (int b : a)
	    if (b<=0) return false;
	return true;
    }
}
