//
// sum(a[i].x[i]) <= c
//

package weeSeepy.constraints;

import weeSeepy.problem.*;
import weeSeepy.variables.*;

public class SumLEQ extends Constraint {

    int c, n;
    int[] a;
    IntVar[] x;

    public SumLEQ(Problem pb, int[] a, IntVar[] x, int c){
	super(pb);
	this.c = c;
	this.a = a;
	this.x = x;
	this.n = x.length;
	name   = "SumLEQ";
	addVariables(x);
	assert allPositive() : "all coeffs must be > 0 "+ this;
    }

    public void propagate(){
	int sumOfMins = 0;
	for (int i=0;i<n;i++) sumOfMins = sumOfMins + a[i] * x[i].min();
	int slack = c - sumOfMins; 
	for (int j=0;j<n;j++){
	    int S_j = slack + a[j] * x[j].min(); // max slack on a[j]*x[j].min()
	    x[j].removeAbove(S_j/a[j]); // can have no more than this!
	}
    }

    public void propagateInit(){
	propagate();
    }
    
    public void propagateRemoval(IntVar x, int value){}
    
    public void propagateUPB(IntVar x){
	propagate();
    }
    
    public void propagateLWB(IntVar x){
	propagate();
    }
    
    public void propagateInstantiate(IntVar x){
	propagate();
    }
    
    public boolean isEntailed(){
	return false;
    }

    public Constraint makeOpposite(){return new Null(pb);}

    public boolean isLegal(){
	return true;
    }

    private boolean allPositive(){
	for (int b : a)
	    if (b<=0) return false;
	return true;
    }
}
