package weeSeepy.squares;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.constraints.Constraint;
import org.chocosolver.solver.search.strategy.Search;
import org.chocosolver.solver.exception.ContradictionException;
import java.util.*;
import java.io.*;

public class ChocoSquares {

    public static void main(String[] args) throws IOException {
	
	Scanner sc = new Scanner(new File(args[0]));
	sc.next(); // pallet
	int maxX = sc.nextInt();
	int maxY = sc.nextInt();
	sc.next(); // n
	int n = sc.nextInt();
	sc.next(); // sizes
	int[] size = new int[n];
	for (int i=0;i<n;i++) size[i] = sc.nextInt();
	sc.close();
	
	Model model   = new Model("ChocoSquares");
	Solver solver = model.getSolver();

	IntVar[] x = new IntVar[n];
	IntVar[] y = new IntVar[n];

	for (int i=0;i<n;i++){
	    x[i] = model.intVar("x_" + i,0,maxX-size[i]);
	    y[i] = model.intVar("y_" + i,0,maxY-size[i]);
	}

	for (int i=0;i<n-1;i++)
	    for (int j=i+1;j<n;j++){
		Constraint[] C = new Constraint[4];
		C[0] = model.arithm(x[j],">=",x[i],"+",size[i]); // square[i] left of square[j]
		C[1] = model.arithm(y[j],">=",y[i],"+",size[i]); // square[i] below square[j]
		C[2] = model.arithm(x[i],">=",x[j],"+",size[j]); // square[i] right of square[j]
		C[3] = model.arithm(y[i],">=",y[j],"+",size[j]); // square[i] above square[j]
		model.or(C).post();
	    }
	
	IntVar[] decVars = new IntVar[2*n];
	for (int i=0,k=0;i<n;i++){
	    decVars[k] = x[i];
	    decVars[k+1] = y[i];
	    k = k + 2;
	}

	solver.setSearch(Search.minDomLBSearch(decVars)); // fail-first
	
	boolean satisfied = solver.solve();
	
	if (satisfied){
	    System.out.println("n "+ n +" maxX "+ maxX +" maxY "+ maxY);
	    for (int i=0;i<n;i++)
		System.out.println(x[i].getValue() +" "+ y[i].getValue() +" "+ size[i]);
	}
	System.out.println(solver.getMeasures());

    }
}
