package weeSeepy.squares;

import weeSeepy.problem.*;
import weeSeepy.variables.*;
import weeSeepy.constraints.*;
import weeSeepy.heuristics.*;

import java.util.*;
import java.io.*;

public class Squares {

    public static void main(String[] args) throws IOException {
	
	Scanner sc = new Scanner(new File(args[0]));
	sc.next(); // pallet
	int maxX = sc.nextInt();
	int maxY = sc.nextInt();
	sc.next(); // n
	int n = sc.nextInt();
	sc.next(); // sizes
	int[] size = new int[n];
	for (int i=0;i<n;i++) size[i] = sc.nextInt();
	sc.close();
	 
	Problem pb = new Problem("Squares");

	IntVar[] x = new IntVar[n];
	IntVar[] y = new IntVar[n];

	for (int i=0;i<n;i++){
	    x[i] = pb.intVar("x_" + i,0,maxX-size[i]);
	    y[i] = pb.intVar("y_" + i,0,maxY-size[i]);
	}

	for (int i=0;i<n-1;i++)
	    for (int j=i+1;j<n;j++){
		Constraint[] C = new Constraint[4];
		C[0] = new GreaterThanOrEqual(pb,x[j],x[i],size[i]); // square[i] left of square[j]
		C[1] = new GreaterThanOrEqual(pb,y[j],y[i],size[i]); // square[i] below square[j]
		C[2] = new GreaterThanOrEqual(pb,x[i],x[j],size[j]); // square[i] right of square[j]
		C[3] = new GreaterThanOrEqual(pb,y[i],y[j],size[j]); // square[i] above square[j]
		new Or(pb,C);
	    }

	pb.trace = args.length > 1;
	
	//
	// NOTE: the x/y variables for squares of large size will habe relatively small
	//       domains, consequently the SDF variable ordering heuristic is leaning
	//       towards the "first fit decreasing" bin packing heuristic!
	//
	ArrayList<IntVar> decVars = new ArrayList<IntVar>();
	for (int i=0;i<n;i++){
	    decVars.add(x[i]);
	    decVars.add(y[i]);
	}
	pb.voh = new SDF(decVars); // smallest domain first

	//
	// NOTE: there are symmetries that might be addressed. In particular, two squares S_i and S_j
	//       where size_i = size_j, might be constrained such that S_i is never right of S_j and
	//       S_i is never above S_j ... but this may make search worse!
	//
	/*
	for (int i=0;i<n-1;i++)
	    for (int j=i+1;j<n;j++)
		if (size[i] == size[j]){
		    pb.post(new GreaterThanOrEqual(pb,x[j],x[i])); // ignore size
		    pb.post(new GreaterThanOrEqual(pb,y[j],y[i])); // ignore size
		}
	*/
    
	boolean satisfied = pb.solve();
	if (satisfied){
	    System.out.println("n "+ n +" maxX "+ maxX +" maxY "+ maxY);
	    for (int i=0;i<n;i++)
		System.out.println(x[i].getValue() +" "+ y[i].getValue() +" "+ size[i]);
	}
	System.out.println("solved: "+ satisfied +" nodes: "+ pb.nodes
			   +"  fails: "+ pb.fails +"  cpuTime: "+ pb.cpuTime);
    }
	
}
