package weeSeepy.variables;

import java.util.*;
import weeSeepy.constraints.*;
import weeSeepy.domain.*;
import weeSeepy.problem.*;
import weeSeepy.exceptions.*;

public class IntVar extends Var implements Iterable<Integer> {
    
    public String name;
    public ArrayList<Constraint> constraints;
    public LinkedList<Integer> deletions;
    Problem pb;
    public Domain domain;    

    public IntVar(String name,int lwb, int upb,Problem pb){
	this.name   = name;
	this.pb     = pb;
	domain      = new Domain(pb,lwb,upb);
	constraints = new ArrayList<Constraint>();
	deletions   = new LinkedList<Integer>();
	pb.variables.add(this);
    }

    public int size(){return domain.size();}

    public boolean contains(int x){return domain.contains(x);}
    
    public boolean remove(int x){
	if (pb.trace) System.out.println("remove: "+ this +" "+ x +" in world: "+ pb.world);
	boolean changed = domain.remove(x);
	if (changed && pb.propagationOn) changeRemoval(x);
	return changed;
    }
    
    public boolean instantiate(int x){
	if (pb.trace) System.out.println("instantiate: "+ this +" "+ x +" in world: "+ pb.world);
	boolean changed = domain.removeAllBut(x);
	if (changed && pb.propagationOn) changeInstantiate();
	return changed;
    }
    
    public boolean isInstantiated(){return domain.size() == 1;}

    public int min(){return domain.min();}

    public int max(){return domain.max();}

    public boolean removeAbove(int x){
	if (pb.trace) System.out.println("removeAbove: "+ this +" "+ x +" in world: "+ pb.world);
	boolean changed = domain.removeAbove(x);
	if (changed && pb.propagationOn) changeUPB();
	return changed;
    }

    public boolean removeBelow(int x){
	if (pb.trace) System.out.println("removeBelow: "+ this +" "+ x +" in world: "+ pb.world);
	boolean changed = domain.removeBelow(x);
	if (changed && pb.propagationOn) changeLWB();
	return changed;
    }

    public int getValue(){
	if (!isInstantiated()) throw new IntVarException("Not instantiated");
	return domain.getLastValue();
    }

    void changeRemoval(int value){	   
	if (size() == 1) changeInstantiate();
	if (value > max()) changeUPB();
	if (value < min()) changeLWB();
	if (min() < value && value < max()){
	    deletions.add(value);
	    for (Constraint c : constraints){
		if (!c.onQueue){
		    pb.revisionQueue.add(c);
		    c.onQueue = true;
		}
		if (!c.removals.contains(this)) c.removals.add(this);
	    }
	}
    }

    void changeInstantiate(){
	for (Constraint c : constraints){
	    if (!c.onQueue){
		pb.revisionQueue.add(c);
		c.onQueue = true;
	    }
	    if (!c.instantiations.contains(this)) c.instantiations.add(this);
	}
    }
	
    void changeUPB(){
	if (size() == 1)
	    changeInstantiate();
	else
	    for (Constraint c : constraints){
		if (!c.onQueue){
		    pb.revisionQueue.add(c);
		    c.onQueue = true;
		}
		if (!c.newUPB.contains(this)) c.newUPB.add(this);
	    }
    }

    void changeLWB(){
	if (size() == 1)
	    changeInstantiate();
	else
	    for (Constraint c : constraints){
		if (!c.onQueue){
		    pb.revisionQueue.add(c);
		    c.onQueue = true;
		}
		if (!c.newLWB.contains(this)) c.newLWB.add(this);
		if (pb.trace) System.out.println("changeLWB push: "+ c);
	    }
    }
    
    public int getLwb(){return domain.lwb;}
    
    public int getUpb(){return domain.upb;}
    
    public String toString(){
	return name +": "+ domain;
    }
    
    public Iterator<Integer> iterator(){
	DomainIterator diter = new DomainIterator(this);
	return (Iterator<Integer>) diter;
    }
}
